package com.android.xmpptest.activity;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.jivesoftware.smack.Chat;
import org.jivesoftware.smack.ChatManager;
import org.jivesoftware.smack.ChatManagerListener;
import org.jivesoftware.smack.MessageListener;
import org.jivesoftware.smack.XMPPException;
import org.jivesoftware.smack.packet.Message;
import org.jivesoftware.smackx.filetransfer.FileTransfer;
import org.jivesoftware.smackx.filetransfer.FileTransferListener;
import org.jivesoftware.smackx.filetransfer.FileTransferManager;
import org.jivesoftware.smackx.filetransfer.FileTransferRequest;
import org.jivesoftware.smackx.filetransfer.IncomingFileTransfer;
import org.jivesoftware.smackx.filetransfer.OutgoingFileTransfer;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.provider.MediaStore;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.android.xmpptest.AppGlobal;
import com.android.xmpptest.BaseActivity;
import com.android.xmpptest.R;
import com.android.xmpptest.entity.Msg;
import com.android.xmpptest.utils.DateTimeUtils;
import com.android.xmpptest.utils.KeyBoardUtils;
import com.android.xmpptest.utils.StringUtils;
import com.android.xmpptest.utils.ToastUtils;
import com.android.xmpptest.xmpp.XmppConnection;

public class ChatActivity extends BaseActivity {

	private static final int TUPIAN_RESULT = 1001;

	private AppGlobal mAppGlobal;

	private TextView tvLeft, tvCenter;

	private String toUserID, toUserName;

	private ListView mListView;

	private ChatAdapter mAdapter;

	private List<Msg> chatList;

	private EditText etSend;

	private Button btnMore, btnSend;

	private ChatManager chatMan;

	private Chat newchat;

	private LinearLayout llFunction;

	private TextView[] tvFunction;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_chat);
		initDatas();
		initTitle();
		initViews();
	}

	private void initDatas() {
		mAppGlobal = (AppGlobal) getApplication();
		Intent intent = getIntent();
		toUserID = intent.getStringExtra("USERID");
		toUserName = intent.getStringExtra("USERNAME");
		// 消息监听
		chatMan = XmppConnection.getConnection().getChatManager();
		newchat = chatMan.createChat(toUserID, null);
		//接受消息监听
		addChatListener();
		//接收文件监听
		addFileListerer();
	}
	
	private void addChatListener(){
		chatMan.addChatListener(new ChatManagerListener() {
			@Override
			public void chatCreated(Chat chat, boolean able) {
				chat.addMessageListener(new MessageListener() {
					@Override
					public void processMessage(Chat chat, Message message) {
						// 收到来自pc服务器的消息（获取自己好友发来的信息）
						if (message.getFrom().contains(toUserID)) {
							if (message.getBody().length() > 0) {
								// 获取用户、消息、时间、IN
								String[] args = new String[] { toUserName, message.getBody() };
								// 在handler里取出来显示消息
								android.os.Message msg = handler.obtainMessage();
								msg.what = 1;
								msg.obj = args;
								msg.sendToTarget();
							}
						}
					}
				});
			}
		});
	}

	private void addFileListerer() {
		FileTransferManager manager = XmppConnection.getFileTransferManager();
		manager.addFileTransferListener(new FileTransferListener() {
			@Override
			public void fileTransferRequest(final FileTransferRequest request) {
				new Thread() {
					@Override
					public void run() {
						IncomingFileTransfer transfer = request.accept();
						String fileName = transfer.getFileName();
						// 在handler里取出来显示消息
						String[] args = new String[] { toUserName, fileName };
						android.os.Message msg = handler.obtainMessage();
						msg.what = 1;
						msg.obj = args;
						msg.sendToTarget();
						
						String save_path = Environment
								.getExternalStorageDirectory()
								+ File.separator
								+ "XmppTest"
								+ File.separator
								+ fileName;
						File file = new File(save_path);
						try {
							transfer.recieveFile(file);
							while (!transfer.isDone()) {
								if (transfer.getStatus().equals(FileTransfer.Status.error)) {
									System.out.println("ERROR!!! " + transfer.getError());
								} else {
									System.out.println(transfer.getStatus());
					                System.out.println(transfer.getProgress());
								}
								try {
									Thread.sleep(1000L);
								} catch (Exception e) {
								}
							}
						} catch (XMPPException e) {  
			                e.printStackTrace();  
			            }  
					};
				}.start();
			}
		});
	}

	private void initTitle() {
		tvLeft = (TextView) findViewById(R.id.tv_header_left);
		tvCenter = (TextView) findViewById(R.id.tv_header_center);
		tvCenter.setText(toUserName);
		tvLeft.setText(R.string.back);
		tvLeft.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				activity.finish();
			}
		});
	}

	private void initViews() {
		mListView = (ListView) findViewById(R.id.lv_chat_list);
		chatList = new ArrayList<Msg>();
		mAdapter = new ChatAdapter(activity, chatList);
		mListView.setAdapter(mAdapter);

		etSend = (EditText) findViewById(R.id.et_send);
		btnMore = (Button) findViewById(R.id.btn_more);
		btnSend = (Button) findViewById(R.id.btn_send);

		etSend.setOnClickListener(mOnClickListener);
		btnMore.setOnClickListener(mOnClickListener);
		btnSend.setOnClickListener(mOnClickListener);
		initFunction();
	}

	OnClickListener mOnClickListener = new OnClickListener() {

		@Override
		public void onClick(View v) {
			switch (v.getId()) {
			case R.id.et_send:
				hideFunction();
				break;
			case R.id.btn_more:
				showFunction();
				break;
			case R.id.btn_send:
				sendMessage();
				break;
			case R.id.tv_send_image:
				Intent intent = new Intent();
				intent.setType("image/*");
				intent.setAction(Intent.ACTION_GET_CONTENT);
				startActivityForResult(intent, TUPIAN_RESULT);
				break;
			case R.id.tv_send_emotion:
				break;
			case R.id.tv_send_file:
				break;
			case R.id.tv_send_video:
				break;
			}
		}
	};

	private void hideFunction() {
		KeyBoardUtils.openKeybord(etSend, activity);
		new Handler().postDelayed(new Runnable() {
			public void run() {
				llFunction.setVisibility(View.GONE);
			}
		}, 200);
	}

	private void showFunction() {
		KeyBoardUtils.closeKeybord(etSend, activity);
		new Handler().postDelayed(new Runnable() {
			public void run() {
				llFunction.setVisibility(View.VISIBLE);
			}
		}, 200);
	}

	private void sendMessage() {
		final String content = etSend.getText().toString();
		if (!StringUtils.isNullOrEmpty(content)) {
			// 发送消息
			String fromUserID = mAppGlobal.getName();
			String dateStr = DateTimeUtils.formatDate(new Date());
			chatList.add(new Msg(dateStr, fromUserID, content, "OUT"));
			// 刷新适配器
			mAdapter.notifyDataSetChanged();
			mListView.setSelection(ListView.FOCUS_DOWN);// 刷新到底部
			Thread thread = new Thread(new Runnable() {

				@Override
				public void run() {
					try {
						Message msg = new Message();
						msg.setBody(content);
						// 发送消息
						newchat.sendMessage(msg);
					} catch (XMPPException e) {
						e.printStackTrace();
					}
				}
			});
			thread.start();
			etSend.setText("");
		} else {
			ToastUtils.showShort(activity, "发送消息为空");
		}
	}

	/**
	 * 初始化发送面板
	 */
	private void initFunction() {
		llFunction = (LinearLayout) findViewById(R.id.ll_send_function);
		tvFunction = new TextView[4];
		tvFunction[0] = (TextView) findViewById(R.id.tv_send_image);
		tvFunction[1] = (TextView) findViewById(R.id.tv_send_emotion);
		tvFunction[2] = (TextView) findViewById(R.id.tv_send_file);
		tvFunction[3] = (TextView) findViewById(R.id.tv_send_video);
		int length = tvFunction.length;
		for (int i = 0; i < length; i++) {
			tvFunction[i].setOnClickListener(mOnClickListener);
		}
	}

	class ChatAdapter extends BaseAdapter {

		private List<Msg> dataList;

		private LayoutInflater mInflater;

		ChatAdapter(Context context, List<Msg> dataList) {
			this.mInflater = LayoutInflater.from(context);
			this.dataList = dataList;
		}

		@Override
		public int getCount() {
			return dataList.size();
		}

		@Override
		public Object getItem(int position) {
			return dataList.get(position);
		}

		@Override
		public long getItemId(int position) {
			return position;
		}

		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			ViewHolder holder = null;
			if (convertView == null) {
				holder = new ViewHolder();
				convertView = mInflater.inflate(R.layout.item_chat_list, null);
				holder.llLeft = (LinearLayout) convertView
						.findViewById(R.id.ll_chat_left);
				holder.llRight = (LinearLayout) convertView
						.findViewById(R.id.ll_chat_right);
				holder.tvDate = (TextView) convertView
						.findViewById(R.id.tv_chat_date);
				holder.tvName = (TextView) convertView
						.findViewById(R.id.tv_chat_name);
				holder.tvTitle = (TextView) convertView
						.findViewById(R.id.tv_chat_title);
				holder.tvName2 = (TextView) convertView
						.findViewById(R.id.tv_chat_name2);
				holder.tvTitle2 = (TextView) convertView
						.findViewById(R.id.tv_chat_title2);
				convertView.setTag(holder);
			} else {
				holder = (ViewHolder) convertView.getTag();
			}
			Msg msg = dataList.get(position);
			holder.tvDate.setText(msg.getDate());
			String myself = msg.getMyself();
			if (myself.equals("IN")) {
				holder.llLeft.setVisibility(View.VISIBLE);
				holder.llRight.setVisibility(View.GONE);
				holder.tvName.setText(msg.getName());
				holder.tvTitle.setText(msg.getTitle());
			} else if (myself.equals("OUT")) {
				holder.llLeft.setVisibility(View.GONE);
				holder.llRight.setVisibility(View.VISIBLE);
				holder.tvName2.setText(msg.getName());
				holder.tvTitle2.setText(msg.getTitle());
			}
			return convertView;
		}

		class ViewHolder {
			LinearLayout llLeft;
			LinearLayout llRight;
			TextView tvDate;
			TextView tvName;
			TextView tvTitle;
			TextView tvName2;
			TextView tvTitle2;
		}
	}

	class SendFileTask extends AsyncTask<String, Integer, Integer> {
		protected Integer doInBackground(String... params) {
			if (params.length < 2) {
				return Integer.valueOf(-1);
			}
			String img_path = params[0];
			String toId = params[1];

			FileTransferManager fileTransferManager = XmppConnection.getFileTransferManager();
			File filetosend = new File(img_path);
			if (filetosend.exists() == false) {
				return -1;
			}
			OutgoingFileTransfer transfer = fileTransferManager
					.createOutgoingFileTransfer(toId);// 创建一个输出文件传输对象
			try {
				transfer.sendFile(filetosend, "recv img");
				while (!transfer.isDone()) {
					if (transfer.getStatus().equals(FileTransfer.Status.error)) {
						System.out.println("ERROR!!! " + transfer.getError());
					} else {
						System.out.println(transfer.getStatus());
		                System.out.println(transfer.getProgress());
					}
					try {
						Thread.sleep(1000);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
			} catch (XMPPException e1) {
				e1.printStackTrace();
			}
			return 0;
		}
	}

	@SuppressLint("HandlerLeak")
	private Handler handler = new Handler() {
		public void handleMessage(android.os.Message msg) {
			String[] args = (String[]) msg.obj;
			String dateStr = DateTimeUtils.formatDate(new Date());
			Msg m = new Msg(dateStr, args[0], args[1], "IN");
			chatList.add(m);
			mAdapter.notifyDataSetChanged();
			mListView.setSelection(ListView.FOCUS_DOWN);// 刷新到底部
		};
	};

	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		Uri uri = data.getData();
		String[] proj = { MediaStore.Images.Media.DATA };
		Cursor cursor = activity.managedQuery(uri, proj, null, null, null);
		int index = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DATA);
		cursor.moveToFirst();
		String img_path = cursor.getString(index);
		new SendFileTask().execute(img_path, toUserID);// 图片路径和我要向
	};
}
