package com.android.xmpptest.activity;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jivesoftware.smack.Roster;
import org.jivesoftware.smack.RosterEntry;
import org.jivesoftware.smack.XMPPConnection;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.view.KeyEvent;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.TextView;

import com.android.xmpptest.BaseActivity;
import com.android.xmpptest.R;
import com.android.xmpptest.utils.AppManager;
import com.android.xmpptest.xmpp.XmppConnection;

public class FriendActivity extends BaseActivity {
	
	private TextView tvLeft, tvCenter;

	private ListView mListView;

	private TextView tvEmpty;

	private List<Map<String,String>> friendList;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_friend);
		initTitle();
		initViews();
		loadDatas();
	}
	
	private void initTitle(){
		tvLeft = (TextView) findViewById(R.id.tv_header_left);
		tvCenter = (TextView) findViewById(R.id.tv_header_center);
		tvLeft.setVisibility(View.GONE);
		tvCenter.setText(R.string.friend_mine);
	}
	
	private void initViews() {
		mListView = (ListView) findViewById(R.id.lv_friend_list);
		tvEmpty = (TextView) findViewById(R.id.tv_friend_empty);
	}
	
	private void refreshViews(){
		if (friendList != null && friendList.size() > 0) {
			mListView.setAdapter(new SimpleAdapter(activity, friendList,
					R.layout.item_friend_list, new String[] { "Name" },
					new int[] { R.id.tv_friend_name }));
			mListView.setOnItemClickListener(mOnItemClickListener);
			mListView.setVisibility(View.VISIBLE);
			tvEmpty.setVisibility(View.GONE);
		} else {
			mListView.setVisibility(View.GONE);
			tvEmpty.setVisibility(View.VISIBLE);
		}
	}
	
	OnItemClickListener mOnItemClickListener = new OnItemClickListener() {

		@Override
		public void onItemClick(AdapterView<?> parent, View view, int position,
				long id) {
			@SuppressWarnings("unchecked")
			Map<String, String> map = (Map<String, String>) parent.getItemAtPosition(position);
			String userID = map.get("User");
			String userName = map.get("Name");
			Intent intent = new Intent(activity, ChatActivity.class);
			intent.putExtra("USERID", userID);
			intent.putExtra("USERNAME", userName);
			activity.startActivity(intent);
		}
	};

	private void loadDatas() {
		Thread thread = new Thread(new Runnable() {
			public void run() {
				try {
					XMPPConnection conn = XmppConnection.getConnection();
					Roster roster = conn.getRoster();
					friendList = new ArrayList<Map<String,String>>();
					Collection<RosterEntry> entries = roster.getEntries();
					HashMap<String, String> map = null;
					for (RosterEntry entry : entries) {
						map = new HashMap<String, String>();
						map.put("User", entry.getUser());
						map.put("Name", entry.getName());
						friendList.add(map);
					}
					handler.sendEmptyMessage(0);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
		thread.start();
	}
	
	@SuppressLint("HandlerLeak")
	private Handler handler = new Handler() {
		public void handleMessage(android.os.Message msg) {
			refreshViews();
		};
	};
	
	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		if (keyCode == KeyEvent.KEYCODE_BACK) {
			XmppConnection.closeConnection();
			AppManager.getAppManager().finishAllActivity();
			return false;
		} else {
			return super.onKeyDown(keyCode, event);
		}
	}
}
